@@ -6,16 +6,6 @@
 
 #include <sys/stat.h>
 #include <errno.h>
-#include <zsv.h>
-#include <zsv/utils/signal.h>
-#include <zsv/utils/arg.h>
-#include <zsv/utils/mem.h>
-#ifndef STRING_LIB_INCLUDE
-#include <zsv/utils/string.h>
-#else
-#include STRING_LIB_INCLUDE
-#endif
-
 #include <unistd.h> // unlink
 #include <stdio.h>
 #include <string.h>
@@ -23,8 +13,16 @@
 
 #include <sqlite3.h>
 
+#define ZSV_COMMAND 2db
+#include "zsv_command.h"
+
+#include <zsv/utils/mem.h>
+#include <zsv/utils/string.h>
+
 #include <yajl_helper.h>
 
+#define ZSV_2DB_DEFAULT_TABLE_NAME "mytable"
+
 enum zsv_2db_action {
   zsv_2db_action_create = 1,
   zsv_2db_action_append,
@@ -163,10 +161,6 @@ static void zsv_2db_delete(zsv_2db_handle data) {
   free(data);
 }
 
-#ifndef MAIN
-#define MAIN main
-#endif
-
 static int zsv_2db_json_parse_err(struct zsv_2db_data *data,
                                   unsigned char *last_parsed_buff,
                                   size_t last_parsed_buff_len
@@ -197,10 +191,10 @@ static int zsv_2db_add_indexes(struct zsv_2db_data *data) {
   int err = 0;
   for(struct zsv_2db_ix *ix = data->json_parser.indexes; !err && ix; ix = ix->next) {
     sqlite3_str *pStr = sqlite3_str_new(data->db);
-    sqlite3_str_appendf(pStr, "create%s index \"%w_ix_%lli\" on \"%w\"(%s)",
+    sqlite3_str_appendf(pStr, "create%s index \"%w_%w\" on \"%w\"(%s)",
                         ix->unique ? " unique" : "",
                         data->opts.table_name,
-                        data->json_parser.index_sequence_num_max,
+                        ix->name,
                         data->opts.table_name,
                         ix->on);
     err = zsv_2db_sqlite3_exec_2db(data->db, sqlite3_str_value(pStr));
@@ -231,7 +225,7 @@ static sqlite3_str *build_create_table_statement(sqlite3 *db, const char *tname,
                                                  ) {
   int err = 0;
   sqlite3_str *pStr = sqlite3_str_new(db);
-  sqlite3_str_appendf(pStr, "CREATE TABLE %s (\n  ", tname);
+  sqlite3_str_appendf(pStr, "CREATE TABLE \"%w\" (\n  ", tname ? tname : ZSV_2DB_DEFAULT_TABLE_NAME);
   for(unsigned int i = 0; i < col_count; i++) {
     if(i > 0)
       sqlite3_str_appendf(pStr, ",\n  ");
@@ -519,60 +513,61 @@ static int json_end_array(struct yajl_helper_parse_state *st) {
 
 static int json_process_value(struct yajl_helper_parse_state *st,
                               struct json_value *value) {
+  const unsigned char *jsstr;
+  size_t len;
   struct zsv_2db_data *data = yajl_helper_data(st);
   if(data->json_parser.state == zsv_2db_state_data) {
     if(yajl_helper_got_path(st, 3, "[[[")) {
-      const unsigned char *src;
-      size_t len;
-      json_value_default_string(value, &src, &len);
-      if(src && len) {
+      json_value_default_string(value, &jsstr, &len);
+      if(jsstr && len) {
         unsigned int j = yajl_helper_array_index_plus_1(st, 0);
         if(j && j-1 < data->json_parser.col_count) {
-          data->json_parser.row_values[j-1] = zsv_memdup(src, len);
+          data->json_parser.row_values[j-1] = zsv_memdup(jsstr, len);
           data->json_parser.have_row_data = 1;
         }
       }
     }
+  } else if(yajl_helper_got_path(st, 2, "[{name")) {
+    json_value_default_string(value, &jsstr, &len);
+    if(len) {
+      if(data->opts.table_name)
+        fprintf(stderr, "Table name specified twice; keeping %s, ignoring %.*s\n",
+                data->opts.table_name, (int)len, jsstr);
+      else
+        data->opts.table_name = zsv_memdup(jsstr, len);
+    }
   } else if(yajl_helper_got_path(st, 4, "[{columns[{name")) {
     free(data->json_parser.current_column.name);
     data->json_parser.current_column.name = NULL;
-    const unsigned char *src;
-    size_t len;
-    json_value_default_string(value, &src, &len);
-    if(src && len)
-      data->json_parser.current_column.name = zsv_memdup(src, len);
+    json_value_default_string(value, &jsstr, &len);
+    if(jsstr && len)
+      data->json_parser.current_column.name = zsv_memdup(jsstr, len);
   } else if(yajl_helper_got_path(st, 4, "[{columns[{datatype")) {
     free(data->json_parser.current_column.datatype);
     data->json_parser.current_column.datatype = NULL;
-    const unsigned char *src;
-    size_t len;
-    json_value_default_string(value, &src, &len);
-    if(src && len)
-      data->json_parser.current_column.datatype = zsv_memdup(src, len);
+    json_value_default_string(value, &jsstr, &len);
+    if(jsstr && len)
+      data->json_parser.current_column.datatype = zsv_memdup(jsstr, len);
   } else if(yajl_helper_got_path(st, 4, "[{columns[{collate")) {
     free(data->json_parser.current_column.collate);
     data->json_parser.current_column.collate = NULL;
-    const unsigned char *src;
-    size_t len;
-    json_value_default_string(value, &src, &len);
-    if(src && len)
-      data->json_parser.current_column.collate = zsv_memdup(src, len);
+    json_value_default_string(value, &jsstr, &len);
+    if(jsstr && len)
+      data->json_parser.current_column.collate = zsv_memdup(jsstr, len);
   } else if(yajl_helper_got_path(st, 4, "[{indexes{*{delete")) {
     data->json_parser.current_index.delete = json_value_truthy(value);
   } else if(yajl_helper_got_path(st, 4, "[{indexes{*{unique")) {
     data->json_parser.current_index.unique = json_value_truthy(value);
   } else if(yajl_helper_got_path(st, 4, "[{indexes{*{on")
             || yajl_helper_got_path(st, 5, "[{indexes{*{on[")) {
-    const unsigned char *src;
-    size_t len;
-    json_value_default_string(value, &src, &len);
+    json_value_default_string(value, &jsstr, &len);
     if(len) {
       if(yajl_helper_level(st) == 4 || !data->json_parser.current_index.on) {
         free(data->json_parser.current_index.on);
-        data->json_parser.current_index.on = zsv_memdup(src, len);
+        data->json_parser.current_index.on = zsv_memdup(jsstr, len);
       } else {
         char *defn;
-        asprintf(&defn, "%s,%.*s", data->json_parser.current_index.on, (int)len, src);
+        asprintf(&defn, "%s,%.*s", data->json_parser.current_index.on, (int)len, jsstr);
         free(data->json_parser.current_index.on);
         data->json_parser.current_index.on = defn;
       }
@@ -605,9 +600,10 @@ static zsv_2db_handle zsv_2db_new(struct zsv_2db_options *opts) {
   data->json_parser.last_column = &data->json_parser.columns;
   data->json_parser.last_index = &data->json_parser.indexes;
   data->json_parser.state = zsv_2db_state_header;
-  data->opts.table_name = strdup(opts->table_name ? opts->table_name : "mytable");
+  if(opts->table_name)
+    data->opts.table_name = strdup(opts->table_name);
   asprintf(&data->db_fn_tmp, "%s.tmp", data->opts.db_fn);
-  if(!data->db_fn_tmp || !data->opts.table_name)
+  if(!data->db_fn_tmp)
     err = 1;
   else {
     int flags = SQLITE_OPEN_READWRITE | SQLITE_OPEN_CREATE | SQLITE_OPEN_FULLMUTEX;
@@ -690,26 +686,17 @@ static yajl_handle zsv_2db_yajl_handle(zsv_2db_handle data) {
   return data->json_parser.handle;
 }
 
-#ifndef APPNAME
-# ifdef ZSV_CLI
-#  define APPNAME "zsv 2db"
-# else
-#  define APPNAME "zsv_2db"
-# endif
-#endif
-
-int MAIN(int argc, const char *argv[]) {
-  INIT_CMD_DEFAULT_ARGS();
+int ZSV_MAIN_FUNC(ZSV_COMMAND)(int argc, const char *argv[], struct zsv_opts *zsv_opts, const char *opts_used) {
+  (void)(zsv_opts);
+  (void)(opts_used);
   FILE *f_in = NULL;
   int err = 0;
-  struct zsv_opts default_opts = zsv_get_default_opts();
-
   struct zsv_2db_options opts = { 0 };
-  opts.verbose = default_opts.verbose;
+  opts.verbose = zsv_get_default_opts().verbose;
 
   const char *usage[] =
     {
-     APPNAME ":  streaming JSON to sqlite3 converter",
+     APPNAME ": convert JSON to sqlite3",
      "",
      "Usage: " APPNAME " -o <output path> [-t <table name>] [input.json]\n",
      "",
@@ -753,7 +740,7 @@ int MAIN(int argc, const char *argv[]) {
       fprintf(stderr, "Input file specified more than once\n"), err = 1;
     else if(!(f_in = fopen(argv[i], "rb")))
       fprintf(stderr, "Unable to open for reading: %s\n", argv[i]), err = 1;
-    else if(!(strlen(argv[i]) > 5 && !zsv_stricmp(argv[i] + strlen(argv[i]) - 5, ".json")))
+    else if(!(strlen(argv[i]) > 5 && !zsv_stricmp((const unsigned char *)argv[i] + strlen(argv[i]) - 5, (const unsigned char *)".json")))
       fprintf(stderr, "Warning: input filename does not end with .json (%s)\n", argv[i]);
   }
 
