@@ -32,13 +32,17 @@ static struct zsv_ext *zsv_ext_new(const char *dl_name, const char *id, char ver
 #include "cli_ini.c"
 
 typedef int (cmd_main)(int argc, const char *argv[]);
+typedef int (zsv_cmd)(int argc, const char *argv[], struct zsv_opts *opts, const char *opts_used);
 typedef int (*cmd_reserved)();
 
 struct builtin_cmd {
   const char *name;
   cmd_main *main;
+  zsv_cmd *cmd;
 };
 
+#include "zsv_main.h"
+
 #define CLI_BUILTIN_DECL(x) int main_ ## x(int argc, const char *argv[])
 
 #define CLI_BUILTIN_DECL_STATIC(x) static int main_ ## x(int argc, const char *argv[])
@@ -50,24 +54,28 @@ CLI_BUILTIN_DECL_STATIC(version);
 CLI_BUILTIN_DECL_STATIC(register);
 CLI_BUILTIN_DECL_STATIC(unregister);
 
-CLI_BUILTIN_DECL(select);
-CLI_BUILTIN_DECL(desc);
-CLI_BUILTIN_DECL(count);
-CLI_BUILTIN_DECL(pretty);
-CLI_BUILTIN_DECL(sql);
-CLI_BUILTIN_DECL(flatten);
-CLI_BUILTIN_DECL(2json);
-CLI_BUILTIN_DECL(2tsv);
-CLI_BUILTIN_DECL(serialize);
-CLI_BUILTIN_DECL(stack);
-CLI_BUILTIN_DECL(2db);
+ZSV_MAIN_DECL(select);
+ZSV_MAIN_DECL(count);
+ZSV_MAIN_DECL(2json);
+ZSV_MAIN_DECL(2tsv);
+ZSV_MAIN_DECL(serialize);
+ZSV_MAIN_DECL(flatten);
+ZSV_MAIN_DECL(pretty);
+ZSV_MAIN_DECL(stack);
+ZSV_MAIN_DECL(desc);
+ZSV_MAIN_DECL(sql);
+ZSV_MAIN_DECL(2db);
+ZSV_MAIN_DECL(echo);
+ZSV_MAIN_NO_OPTIONS_DECL(prop);
+ZSV_MAIN_NO_OPTIONS_DECL(rm);
 
 #ifdef USE_JQ
-CLI_BUILTIN_DECL(jq);
+ZSV_MAIN_NO_OPTIONS_DECL(jq);
 #endif
 
-
-#define CLI_BUILTIN_CMD(x) { .name = #x, .main = main_ ## x }
+#define CLI_BUILTIN_CMD(x) { .name = #x, .main = main_ ## x, .cmd = NULL }
+#define CLI_BUILTIN_COMMAND(x) { .name = #x, .main = NULL, .cmd = ZSV_MAIN_FUNC(x) }
+#define CLI_BUILTIN_NO_OPTIONS_COMMAND(x) { .name = #x, .main = ZSV_MAIN_NO_OPTIONS_FUNC(x), .cmd = NULL }
 struct builtin_cmd builtin_cmds[] = {
   CLI_BUILTIN_CMD(license),
   CLI_BUILTIN_CMD(thirdparty),
@@ -76,19 +84,22 @@ struct builtin_cmd builtin_cmds[] = {
   CLI_BUILTIN_CMD(register),
   CLI_BUILTIN_CMD(unregister),
 
-  CLI_BUILTIN_CMD(select),
-  CLI_BUILTIN_CMD(desc),
-  CLI_BUILTIN_CMD(count),
-  CLI_BUILTIN_CMD(pretty),
-  CLI_BUILTIN_CMD(sql),
-  CLI_BUILTIN_CMD(flatten),
-  CLI_BUILTIN_CMD(2json),
-  CLI_BUILTIN_CMD(2tsv),
-  CLI_BUILTIN_CMD(serialize),
-  CLI_BUILTIN_CMD(stack),
-  CLI_BUILTIN_CMD(2db)
+  CLI_BUILTIN_COMMAND(select),
+  CLI_BUILTIN_COMMAND(count),
+  CLI_BUILTIN_COMMAND(2json),
+  CLI_BUILTIN_COMMAND(2tsv),
+  CLI_BUILTIN_COMMAND(serialize),
+  CLI_BUILTIN_COMMAND(flatten),
+  CLI_BUILTIN_COMMAND(pretty),
+  CLI_BUILTIN_COMMAND(stack),
+  CLI_BUILTIN_COMMAND(desc),
+  CLI_BUILTIN_COMMAND(sql),
+  CLI_BUILTIN_COMMAND(2db),
+  CLI_BUILTIN_COMMAND(echo),
+  CLI_BUILTIN_NO_OPTIONS_COMMAND(prop),
+  CLI_BUILTIN_NO_OPTIONS_COMMAND(rm)
 #ifdef USE_JQ
-  , CLI_BUILTIN_CMD(jq)
+  , CLI_BUILTIN_NO_OPTIONS_COMMAND(jq)
 #endif
 };
 
@@ -279,7 +290,7 @@ static enum zsv_ext_status ext_parse_all(zsv_execution_context ctx,
                                          ) {
   struct zsv_opts opts = custom ? *custom : ext_parser_opts(ctx);
   if(row_handler)
-    opts.row = row_handler;
+    opts.row_handler = row_handler;
   zsv_parser parser = zsv_new(&opts);
   if(!parser)
     return zsv_ext_status_memory;
@@ -307,7 +318,7 @@ static struct zsv_ext_callbacks *zsv_ext_callbacks_init(struct zsv_ext_callbacks
     e->set_context = zsv_set_context;
     e->parse_more = zsv_parse_more;
     e->abort = zsv_abort;
-    e->column_count = zsv_column_count;
+    e->cell_count = zsv_cell_count;
     e->get_cell = zsv_get_cell;
     e->finish = zsv_finish;
     e->delete = zsv_delete;
@@ -381,8 +392,14 @@ static enum zsv_ext_status run_extension(int argc, const char *argv[], struct zs
     }
 
     struct zsv_execution_data ctx;
-    if((stat = execution_context_init(&ctx, argc, argv)) == zsv_ext_status_ok)
+
+    if((stat = execution_context_init(&ctx, argc, argv)) == zsv_ext_status_ok) {
+      struct zsv_opts opts;
+      zsv_args_to_opts(argc, argv, &argc, argv, &opts, NULL);
+      zsv_set_default_opts(opts);
       stat = cmd->main(&ctx, ctx.argc - 1, &ctx.argv[1]);
+    }
+
     if(stat != zsv_ext_status_ok)
       stat = handle_ext_err(ext, &ctx, stat);
     execution_context_free(&ctx);
@@ -400,8 +417,8 @@ char havearg(const char *arg,
   if(len > min_len1)
     min_len1 = len;
 
-  if(!zsv_strincmp((const unsigned char *)arg, min_len1,
-                     (const unsigned char *)form1, min_len1))
+  if(!zsv_strincmp_ascii((const unsigned char *)arg, min_len1,
+			 (const unsigned char *)form1, min_len1))
     return 1;
 
   if(form2) {
@@ -409,8 +426,8 @@ char havearg(const char *arg,
       min_len2 = strlen(form2);
     if(len > min_len2)
       min_len2 = len;
-    if(!zsv_strincmp((const unsigned char *)arg, min_len2,
-                       (const unsigned char *)form2, min_len2))
+    if(!zsv_strincmp_ascii((const unsigned char *)arg, min_len2,
+			   (const unsigned char *)form2, min_len2))
       return 1;
   }
   return 0;
@@ -436,14 +453,12 @@ static const char *extension_cmd_from_arg(const char *arg) {
   return NULL;
 }
 
-#ifndef CLI_MAIN
-#define CLI_MAIN main
+#ifndef ZSV_CLI_MAIN
+#define ZSV_CLI_MAIN main
 #endif
 
 ZSV_CLI_EXPORT
-int CLI_MAIN(int argc, const char *argv[]) {
-  INIT_DEFAULT_ARGS();
-
+int ZSV_CLI_MAIN(int argc, const char *argv[]) {
   const char **alt_argv = NULL;
   struct builtin_cmd *builtin = find_builtin(argc > 1 ? argv[1] : "help");
   if(builtin) {
@@ -473,8 +488,17 @@ int CLI_MAIN(int argc, const char *argv[]) {
           return 1;
         }
       }
-    } else
-      return builtin->main(argc - 1, argc > 1 ? &argv[1] : NULL);
+    } else {
+      if(builtin->main)
+        return builtin->main(argc - 1, argc > 1 ? &argv[1] : NULL);
+
+      char opts_used[ZSV_OPTS_SIZE_MAX];
+      struct zsv_opts opts;
+      enum zsv_status stat = zsv_args_to_opts(argc, argv, &argc, argv, &opts, opts_used);
+      if(stat == zsv_status_ok)
+        return builtin->cmd(argc - 1, argc > 1 ? &argv[1] : NULL, &opts, opts_used);
+      return stat;
+    }
   }
 
   int err = 1;
@@ -612,7 +636,7 @@ static struct zsv_ext *zsv_ext_new(const char *dl_name, const char *id, char ver
     } else
       fprintf(stderr, "Opening whaa???\n");
 #else
-    size_t n = get_executable_path(exe_path, sizeof(exe_path));
+    size_t n = zsv_get_executable_path(exe_path, sizeof(exe_path));
     if(n > 0 && n < sizeof(exe_path)) {
       char *end = strrchr(exe_path, FILESLASH);
       if(end) {
