@@ -5,20 +5,6 @@
  * https://opensource.org/licenses/MIT
  */
 
-#include <zsv.h>
-#include <zsv/utils/writer.h>
-#include <zsv/utils/signal.h>
-#include <zsv/utils/file.h>
-#include <zsv/utils/utf8.h>
-#include <zsv/utils/mem.h>
-#include <zsv/utils/arg.h>
-
-#ifndef STRING_LIB_INCLUDE
-#include <zsv/utils/string.h>
-#else
-#include STRING_LIB_INCLUDE
-#endif
-
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -26,10 +12,14 @@
 
 #include <unistd.h> // unlink
 
-#ifndef APPNAME
-#define APPNAME "flatten"
-#endif
-#include <zsv/utils/err.h>
+#define ZSV_COMMAND flatten
+#include "zsv_command.h"
+
+#include <zsv/utils/writer.h>
+#include <zsv/utils/file.h>
+#include <zsv/utils/utf8.h>
+#include <zsv/utils/mem.h>
+#include <zsv/utils/string.h>
 
 enum flatten_agg_method {
   flatten_agg_method_none = 1,
@@ -101,12 +91,13 @@ static void flatten_agg_col_iterator_init(struct flatten_agg_col *c,
   }
 }
 
-static void flatten_agg_col_iterator_replace_str(struct flatten_agg_col_iterator *i, unsigned char *new_s) {
+static void flatten_agg_col_iterator_replace_str(struct flatten_agg_col_iterator *i, unsigned char **new_s) {
   if(i->current_cl)
-    i->current_cl->value = new_s;
+    i->current_cl->value = *new_s;
   else {
     fprintf(stderr, "flatten_agg_col_iterator_replace_str() error: no current value to replace\n");
-    free(new_s);
+    free(*new_s);
+    *new_s = NULL;
   }
 }
 
@@ -197,6 +188,7 @@ struct flatten_data {
   const char *output_filename;
 
   FILE *in;
+  const char *input_path;
   FILE *out;
 
   zsv_csv_writer csv_writer;
@@ -209,9 +201,6 @@ struct flatten_data {
 
   enum flatten_agg_method all_aggregation_method;
 
-  char *overflowed;
-  size_t overflow_count;
-
   unsigned char cancelled:1;
   unsigned char verbose:1;
   unsigned char have_agg:1;
@@ -300,16 +289,6 @@ static void flatten_cell1(void *hook, unsigned char *utf8_value, size_t len) {
   data->current_column_index++;
 }
 
-static void flatten_overflow(void *hook, unsigned char *utf8_value, size_t len) {
-  (void)(hook);
-  struct flatten_data *data = hook;
-  if(!data->overflowed)
-    data->overflowed = zsv_memdup(utf8_value, len);
-  data->overflow_count++;
-  if(data->cancelled)
-    return;
-}
-
 static void flatten_row1(void *hook) {
   struct flatten_data *data = hook;
   if(data->cancelled)
@@ -431,7 +410,8 @@ static void output_current_row(struct flatten_data *data) {
           flatten_agg_col_iterator_next(&it), i++) {
         if(i)
           joined_len += delimiter_len;
-        flatten_agg_col_iterator_replace_str(&it, flatten_replace_delim(it.str, delimiter, replacement));
+        it.str = flatten_replace_delim(it.str, delimiter, replacement);
+        flatten_agg_col_iterator_replace_str(&it, &it.str);
         if(it.str && *it.str)
           joined_len += strlen((char *)it.str);
       }
@@ -515,17 +495,6 @@ static void flatten_row2(void *hook) {
   data->row_count2++;
 }
 
-static void flatten_error(void *hook, enum zsv_status status,
-                          const unsigned char *err_msg, size_t err_msg_len,
-                          unsigned char bad_c, size_t cum_scanned_length) {
-  struct flatten_data *data = hook;
-  if(data->cancelled)
-    return;
-  (void)(status);
-  (void)(err_msg_len);
-  zsv_printerr(1, "%s (%c) at %zu", err_msg, bad_c, cum_scanned_length);
-}
-
 const char *flatten_usage_msg[] =
   {
    APPNAME ": flatten a table, based on a single-column key assuming that rows to flatten always appear in contiguous blocks",
@@ -589,7 +558,7 @@ static struct flatten_agg_col *flatten_agg_col_new(const char *arg, int *err) {
 
   unsigned char *agg_method_s = NULL;
 
-  while(*read) {
+  while(read && *read) {
     if(*read == '=') { // end of name!
       *read = '\0';
       agg_method_s = read + 1;
@@ -659,27 +628,15 @@ static void flatten_cleanup(struct flatten_data *data) {
   }
 
   FREEIF(data->agg_output_cols_vector);
-  if(data->overflowed) {
-    zsv_printerr(1, "Warning: data overflowed %zu times (example: %s)",
-             data->overflow_count, data->overflowed);
-    free(data->overflowed);
-  }
-
   zsv_writer_delete(data->csv_writer);
 }
 
-#ifndef MAIN
-#define MAIN main
-#endif
-
-int MAIN(int argc, const char *argv[]) {
+int ZSV_MAIN_FUNC(ZSV_COMMAND)(int argc, const char *argv[], struct zsv_opts *opts, const char *opts_used) {
   if(argc > 1 && (!strcmp(argv[1], "-h") || !strcmp(argv[1], "--help"))) {
     flatten_usage();
     return 0;
   }
 
-  INIT_CMD_DEFAULT_ARGS();
-
   struct flatten_data data = { 0 };
   struct zsv_csv_writer_options writer_opts = zsv_writer_get_default_opts();
 
@@ -738,6 +695,8 @@ int MAIN(int argc, const char *argv[]) {
       err = zsv_printerr(1, "Input file was specified, cannot also read: %s", argv[arg_i]);
     else if(!(data.in = fopen(argv[arg_i], "rb")))
       err = zsv_printerr(1, "Could not open for reading: %s", argv[arg_i]);
+    else
+      data.input_path = argv[arg_i];
   }
 
   if(!data.in) {
@@ -769,14 +728,8 @@ int MAIN(int argc, const char *argv[]) {
   if(!(data.out = data.output_filename ? fopen(data.output_filename, "wb") : stdout))
     err = zsv_printerr(1, "Unable to open %s for writing", data.output_filename);
 
-  if(err) {
-    flatten_cleanup(&data);
-    return 1;
-  }
-
   int passes = data.column_name_column.name || !data.have_agg ? 2 : 1;
-  struct zsv_opts opts = zsv_get_default_opts();
-
+  const char *input_path = NULL;
   FILE *in = NULL;
   char *tmp_fn = NULL;
   zsv_handle_ctrl_c_signal();
@@ -786,15 +739,14 @@ int MAIN(int argc, const char *argv[]) {
     tmp_fn = zsv_get_temp_filename("zsv_flatten_XXXXXXXX");
     if(tmp_fn) {
       FILE *tmp_f = fopen(tmp_fn, "w+b");
-      opts.cell = flatten_cell1;
-      opts.row = flatten_row1;
-      opts.overflow = flatten_overflow;
-      opts.error = flatten_error;
-      opts.stream = data.in;
-      opts.ctx = &data;
+      opts->cell_handler = flatten_cell1;
+      opts->row_handler = flatten_row1;
+      opts->stream = data.in;
+      input_path = data.input_path;
+      opts->ctx = &data;
 
-      zsv_parser handle = zsv_new(&opts);
-      if(!handle)
+      zsv_parser handle;
+      if(zsv_new_with_properties(opts, input_path, opts_used, &handle) != zsv_status_ok)
         err = data.cancelled = zsv_printerr(1, "Unable to create csv parser");
       else {
         zsv_set_scan_filter(handle, zsv_filter_write, tmp_f);
@@ -812,12 +764,10 @@ int MAIN(int argc, const char *argv[]) {
   }
 
   if(!err) {
-    memset(&opts, 0, sizeof(opts));
-    opts.cell = flatten_cell2;
-    opts.row = flatten_row2;
-    opts.overflow = flatten_overflow;
-    opts.error = flatten_error;
-    opts.ctx = &data;
+    struct zsv_opts opts2 = { 0 };
+    opts2.cell_handler = flatten_cell2;
+    opts2.row_handler = flatten_row2;
+    opts2.ctx = &data;
     data.current_column_index = 0;
 
     if(!(data.csv_writer = zsv_writer_new(&writer_opts)))
@@ -825,8 +775,8 @@ int MAIN(int argc, const char *argv[]) {
 
     flatten_output_header(&data);
 
-    opts.stream = in;
-    zsv_parser parser = zsv_new(&opts);
+    opts2.stream = in;
+    zsv_parser parser = zsv_new(&opts2);
     if(!parser)
       err = data.cancelled = zsv_printerr(1, "Unable to create csv parser");
 
