@@ -5,29 +5,16 @@
  * https://opensource.org/licenses/MIT
  */
 
-#include <zsv.h>
-#include <zsv/utils/writer.h>
-#include <zsv/utils/signal.h>
-#include <zsv/utils/arg.h>
-
 #include <utf8proc.h>
-
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <math.h>
 
-#ifndef STRING_LIB_INCLUDE
+#define ZSV_COMMAND pretty
+#include "zsv_command.h"
+#include <zsv/utils/writer.h>
 #include <zsv/utils/string.h>
-#else
-#include STRING_LIB_INCLUDE
-#endif
-
-#ifndef APPNAME
-#define APPNAME "pretty"
-#endif
-
-#include <zsv/utils/err.h>
 
 #define ZSV_PRETTY_DEFAULT_LINE_MAX_WIDTH 160
 #define ZSV_PRETTY_DEFAULT_COLUMN_MAX_WIDTH 35
@@ -128,7 +115,7 @@ static void zsv_cached_rows_delete(struct zsv_cached_row *first) {
 }
 
 static struct zsv_cached_row *zsv_pretty_dupe_row(zsv_parser parser) {
-  unsigned col_count = zsv_column_count(parser);
+  unsigned col_count = zsv_cell_count(parser);
   if(col_count) {
     struct zsv_cached_row *r = calloc(1, sizeof(*r));
     if(r) {
@@ -204,7 +191,7 @@ static size_t zsv_pretty_get_width(struct zsv_pretty_data *data, size_t ix) {
 
 static size_t zsv_pretty_get_row_count(struct zsv_pretty_data *data,
                                          struct zsv_cached_row *r) {
-  return r ? r->count : zsv_column_count(data->parser);
+  return r ? r->count : zsv_cell_count(data->parser);
 }
 
 static void zsv_pretty_output_lineend(struct zsv_pretty_data *data,
@@ -426,8 +413,7 @@ static enum zsv_pretty_status zsv_pretty_add_to_cache(struct zsv_pretty_data *da
 }
 
 static enum zsv_pretty_status zsv_pretty_update_column_widths(struct zsv_pretty_data *data) {
-  size_t columns_used = zsv_column_count(data->parser);
-//  size_t columns_used = zsv_row_cells_count(r);
+  size_t columns_used = zsv_cell_count(data->parser);
   if(columns_used > 1) {
     if(columns_used > data->widths.allocated) {
       size_t *new_width_values = realloc(data->widths.values, columns_used * sizeof(*new_width_values));
@@ -472,7 +458,7 @@ static void zsv_pretty_row(void *ctx) {
   if(data->err)
     return;
 
-  unsigned int columns_used = zsv_column_count(data->parser);
+  unsigned int columns_used = zsv_cell_count(data->parser);
   if(columns_used < 2 && (zsv_pretty_get_cell(data->parser, NULL, 0).len == 0 || data->widths.used == 0)) {
     zsv_pretty_reset_column_widths(data);
     zsv_pretty_output_row(data, NULL);
@@ -506,12 +492,12 @@ const char *zsv_pretty_usage_msg[] =
    "",
    "Options:",
    "  -o, --output <filename> : save output the the specified file",
-   "  -w, --width <n>: set the max line width to output. if not provided",
+   "  -W, --width <n>: set the max line width to output. if not provided",
    "                            will try to detect automatically",
-   "  -r, --rows:             : set the number of lines to calculated widths from",
+   "  -p, --rows:             : set the number of (preview) rows to calculated widths from",
    "                            if not provided, defaults to 150",
-   "  -c, --max-col-width <n> : set the max column width. if not provided, defaults to 50",
-   "  -C, --min-col-width <n> : set the min column width. if not provided, defaults to 2",
+   "  -C, --max-col-width <n> : set the max column width. if not provided, defaults to 50",
+   "  -D, --min-col-width <n> : set the min column width. if not provided, defaults to 2",
    "  -A, --no-align          : do not fill in additional cells if the first row has more columns than the current row",
    "  -m, --markdown          : output the table in markdown format",
    "  -M, --markdown-pad      : output the table in markdown format with padding",
@@ -547,13 +533,10 @@ static void zsv_pretty_destroy(struct zsv_pretty_data *data) {
   free(data);
 }
 
-#ifndef MAIN
-#define MAIN main
-#endif
-
-static struct zsv_pretty_data * zsv_pretty_init(struct zsv_pretty_opts *opts,
-                                                FILE *in,
-                                                struct zsv_opts *parser_opts) {
+static struct zsv_pretty_data *zsv_pretty_init(struct zsv_pretty_opts *opts,
+                                               struct zsv_opts *parser_opts,
+                                               const char *input_path,
+                                               const char *opts_used) {
   struct zsv_pretty_data *data = calloc(1, sizeof(*data));
   if(!data)
     return NULL;
@@ -568,10 +551,9 @@ static struct zsv_pretty_data * zsv_pretty_init(struct zsv_pretty_opts *opts,
   else if(!(data->line.max = get_console_width()))
     data->line.max = ZSV_PRETTY_DEFAULT_LINE_MAX_WIDTH;
 
-  parser_opts->row = zsv_pretty_row;
+  parser_opts->row_handler = zsv_pretty_row;
   parser_opts->ctx = data;
-  parser_opts->stream = in;
-  data->parser = zsv_new(parser_opts);
+  zsv_new_with_properties(parser_opts, input_path, opts_used, &data->parser);
 
   data->write = (size_t (*)(const void *, size_t, size_t, void *))fwrite;
   data->write_arg = opts->out ? opts->out : stdout;
@@ -596,23 +578,24 @@ static struct zsv_pretty_data * zsv_pretty_init(struct zsv_pretty_opts *opts,
   return data;
 }
 
-int MAIN(int argc, const char *argv[]) {
-  int rc = 0;
-  FILE *in = stdin;
+int ZSV_MAIN_FUNC(ZSV_COMMAND)(int argc, const char *argv[], struct zsv_opts *parser_opts, const char *opts_used) {
   if(argc > 1 && (!strcmp(argv[1], "-h") || !strcmp(argv[1], "--help"))) {
     zsv_pretty_usage();
     return 0;
   }
 
+  int rc = 0;
+  FILE *in = stdin;
+  const char *input_path = NULL;
+
   struct zsv_pretty_opts opts = { 0 };
-  INIT_CMD_DEFAULT_ARGS();
 
   opts.line_width_max = ZSV_PRETTY_DEFAULT_LINE_MAX_WIDTH;
   opts.column_width_min = ZSV_PRETTY_DEFAULT_COLUMN_MIN_WIDTH;
   opts.column_width_max = ZSV_PRETTY_DEFAULT_COLUMN_MAX_WIDTH;
   opts.cache_max_rows = ZSV_PRETTY_DEFAULT_CACHE_MAX;
 
-  const char *size_t_args[] = {"-w", "--width", "-c", "--max-col-width", "-C", "--min-col-width", "-r", "--rows", NULL };
+  const char *size_t_args[] = {"-W", "--width", "-C", "--max-col-width", "-D", "--min-col-width", "-p", "--rows", NULL };
   size_t size_t_maximums[] = { 32000, 32000, 500, 500, 500, 500, 100000000, 100000000 };
   for(int i = 1; !rc && i < argc; i++) {
     if(*argv[i] == '-') {
@@ -669,6 +652,8 @@ int MAIN(int argc, const char *argv[]) {
       }
     } else if(!(in = fopen(argv[i], "rb")))
       rc = zsv_printerr(1, "Unable to open file %s for reading", argv[i]);
+    else
+      input_path = argv[i];
   }
 
 #ifdef NO_STDIN
@@ -678,27 +663,24 @@ int MAIN(int argc, const char *argv[]) {
   if(opts.column_width_min > opts.column_width_max || opts.column_width_min > opts.line_width_max)
     rc = zsv_printerr(1, "Min column width cannot exceed max column width or max line width");
 
-  struct zsv_opts parser_opts = zsv_get_default_opts();
-  if(!rc) {
-    struct zsv_pretty_data *h = zsv_pretty_init(&opts, in, &parser_opts);
-    if(!h)
-      rc = 1;
-    else {
-      zsv_handle_ctrl_c_signal();
-      rc = 0;
-      enum zsv_status status;
-      while(zsv_parse_more(h->parser) == zsv_status_ok)
-        ;
+  parser_opts->stream = in;
+  struct zsv_pretty_data *h = zsv_pretty_init(&opts, parser_opts, input_path, opts_used);
+  if(!h)
+    rc = 1;
+  else {
+    zsv_handle_ctrl_c_signal();
+    rc = 0;
+    enum zsv_status status;
+    while(zsv_parse_more(h->parser) == zsv_status_ok)
+      ;
 
-      while(!rc && !zsv_signal_interrupted
-            && (status = zsv_parse_more(h->parser)) == zsv_status_ok)
-        ;
-    }
+    while(!rc && !zsv_signal_interrupted
+          && (status = zsv_parse_more(h->parser)) == zsv_status_ok)
+      ;
 
     zsv_pretty_flush(h);
     zsv_pretty_destroy(h);
   }
-
   if(opts.out)
     fclose(opts.out);
   if(in && in != stdin)
